DEFINT A-Z

' $INCLUDE: 'KEYCODE.INC'
' $INCLUDE: 'PARM.INC'
' $INCLUDE: 'SAVEENV.INC'
' $INCLUDE: 'SETCURS.INC'
' $INCLUDE: 'TRUEFALS.INC'

DECLARE FUNCTION Dialog (msg$(), choice$(), parm())
DECLARE SUB InfoBox (msg$(), parm())
DECLARE SUB InfoBox2 (msg$, parm())
DECLARE FUNCTION YesNo (msg$(), yesword$, noword$, parm())
DECLARE FUNCTION YesNo2 (msg$, yesword$, noword$, parm())

'External procedures:

DECLARE SUB BoxCalc (t, l, b, r, tall, wide)
DECLARE FUNCTION GetKey$ (parm())
DECLARE SUB PopBox (t, l, b, r, wide, msg$(), parm())

FUNCTION Dialog (msg$(), choice$(), parm())
'****************************************************************************
'Allows the user to pick from a horizontal light-bar menu within a pop-up
' message box.
'
'The informational text of the box is contained within the msg$() array.
'
'The choice$() array contains the items the user may pick from.  The function
' will return the element number of the item selected, or zero if the user
' presses ESC.  For this reason it is suggested that the array elements begin
' with 1.
'
'Also, the user can press the first letter of each choice to make their
' selection.  If a choice's first letter is already taken, the second letter
' will be used.  If the second letter is already in use, the third letter of
' the choice will be used, and so on.  This ability can be turned on or off
' with parm(6).  The letter keys are NOT case sensitive.
'
'    parm(1) = top left row  0=Center
'    parm(2) = top left column  0=Center
'    parm(3) = box border type 1-4.  See SUB Panes() for numeric boxtypes.
'    parm(4) = message justification  <0=Left  0=Center  >0=Right
'    parm(5) = initial selected choice
'    parm(6) = allow letter keys?  0=No  Non-zero=Yes
'
'If a combination of any of the above parameters causes a portion of the box
' to exceed the screen boundaries, a run-time error will occur.
'
'****************************************************************************

'                     *** Preliminary Calculations ***

minc = LBOUND(choice$)                  'Get info about the choice array.
maxc = UBOUND(choice$)

wide = 0: tall = 0                      'Find out how wide and tall to make
FOR x = LBOUND(msg$) TO UBOUND(msg$)    'the box.  Use either the longest
     l = LEN(msg$(x))                   'message or the combined width of all
     IF l > wide THEN wide = l          'the choices to measure the width.
     tall = tall + 1
NEXT x
tall = tall + 2                         'Allow for a blank line & choices.
l = 0
FOR x = minc TO maxc
     l = l + LEN(choice$(x)) + 1        'Allow for spaces between choices.
NEXT x
l = l - 1
IF l > wide THEN wide = l

row1 = parm(1)                          'Calculate where to place the box.
col1 = parm(2)
BoxCalc row1, col1, row2, col2, tall, wide

IF l = wide THEN                        'Calculate the column & row at which
     ccol = col1 + 1                    'the choices will begin.
ELSE
     ccol = col1 + 1 + ((wide - l) \ 2)
END IF
crow = row2 - 1

IF parm(6) THEN                         'Set up the letter keys if wanted.
     FOR x = minc TO maxc
          c$ = UCASE$(LTRIM$(RTRIM$(choice$(x))))
          l = 1
          DO
               l$ = MID$(c$, l, 1)
               l = l + 1
          LOOP UNTIL INSTR(ltrs$, l$) = 0 OR l > LEN(c)
          ltrs$ = ltrs$ + l$
     NEXT x
END IF


'                          *** Draw the Box ***

DIM OldEnv AS EnvironType               'Save the current environment.
SaveEnv OldEnv
x = SetCursor(SCNONE)                   'Turn off the cursor.
PopBox row1, col1, row2, col2, wide, msg$(), parm()


'                     *** Pick one of the choices ***

COLOR parm(FGWT), parm(BGWT)
sel = parm(5)

DO                                      'The main loop to pick a choice.

     IF sel < minc THEN sel = maxc
     IF sel > maxc THEN sel = minc

     LOCATE crow, ccol                  'Print the choices.
     FOR x = minc TO maxc
          IF x = sel THEN COLOR parm(FGWS), parm(BGWS)
          PRINT choice$(x);
          COLOR parm(FGWT), parm(BGWT)
          IF x < maxc THEN PRINT " ";
     NEXT x

     l$ = UCASE$(GetKey$(parm()))

     SELECT CASE KeyCode(l$)
          CASE KCESC                                   'ESC
               Dialog = 0
               EXIT DO
          CASE KCENTER                                 'Enter
               Dialog = sel
               EXIT DO
          CASE KCLEFT, KCUP                            'Left/Up Arrow
               sel = sel - 1
          CASE KCRIGHT, KCDOWN                         'Right/Down Arrow
               sel = sel + 1
          CASE ELSE                                    'Letter key?
               IF parm(6) THEN
                    l = INSTR(ltrs$, l$)
                    IF l THEN
                         Dialog = minc + l - 1
                         EXIT DO
                    END IF
               END IF
     END SELECT

LOOP

'                     *** Clean up after ourselves ***

RestEnv OldEnv

END FUNCTION

SUB InfoBox (msg$(), parm())
'****************************************************************************
'Displays the text of the msg$() array in a pop-up box.  Basically, it is
' just a call to Dialog() with only one option of " Ok ".
'
'    parm(1) = top left row  0=Center
'    parm(2) = top left column  0=Center
'    parm(3) = box border type 1-4.  See SUB Panes() for numeric boxtypes.
'    parm(4) = message justification  <0=Left  0=Center  >0=Right
'    parm(5) = 0
'    parm(6) = 0
'
'See function Dialog() for more detailed information.
'
'****************************************************************************

REDIM choice$(1 TO 1)
choice$(1) = " Ok "

x = Dialog(msg$(), choice$(), parm())

ERASE choice$

END SUB

SUB InfoBox2 (msg$, parm())
'****************************************************************************
'Works just like InfoBox() but accepts a single text string rather than an
' array.
'
'    parm(1) = top left row  0=Center
'    parm(2) = top left column  0=Center
'    parm(3) = box border type 1-4.  See SUB Panes() for numeric boxtypes.
'    parm(4) = message justification  <0=Left  0=Center  >0=Right
'    parm(5) = 0
'    parm(6) = 0
'
'See functions InfoBox() and Dialog() for more detailed information.
'
'****************************************************************************

REDIM msg$(1 TO 1)
msg$(1) = msg$

REDIM choice$(1 TO 1)
choice$(1) = " Ok "

x = Dialog(msg$(), choice$(), parm())

ERASE msg$
ERASE choice$

END SUB

FUNCTION YesNo (msg$(), yesword$, noword$, parm())
'****************************************************************************
'Works like Dialog() but returns TRUE if the yes option is selected or FALSE
' if the no option is selected or ESC is pressed.
'
'    parm(1) = top left row  0=Center
'    parm(2) = top left column  0=Center
'    parm(3) = box border type 1-4.  See SUB Panes() for numeric boxtypes.
'    parm(4) = message justification  <0=Left  0=Center  >0=Right
'    parm(5) = initial selected choice as TRUE or FALSE
'    parm(6) = allow letter keys?  0=No  Non-zero=Yes
'
'The function defaults to the words " Yes " and " No ".  If these are what
' you want, pass null strings for the optional words.  Common alternatives
' might be " Ok " and " Cancel ".
'
'See function Dialog() for more detailed information.
'
'****************************************************************************

REDIM choice$(-1 TO 0)                  'Notice how we trick Dialog() into
IF LEN(yesword$) THEN                   'returning TRUE/FALSE values by
     choice$(-1) = yesword$             'creating an array with the proper
ELSE                                    'subscript values.
     choice$(-1) = " Yes "
END IF
IF LEN(noword$) THEN
     choice$(0) = noword$
ELSE
     choice$(0) = " No "
END IF

YesNo = Dialog(msg$(), choice$(), parm())

ERASE choice$

END FUNCTION

FUNCTION YesNo2 (msg$, yesword$, noword$, parm())
'****************************************************************************
'Works like YesNo() but accepts a single message string rather than an array.
'
'    parm(1) = top left row  0=Center
'    parm(2) = top left column  0=Center
'    parm(3) = box border type 1-4.  See SUB Panes() for numeric boxtypes.
'    parm(4) = message justification  <0=Left  0=Center  >0=Right
'    parm(5) = initial selected choice as TRUE or FALSE
'    parm(6) = allow letter keys?  0=No  Non-zero=Yes
'
'See functions YesNo() and Dialog() for more detailed information.
'
'****************************************************************************

REDIM msg$(1 TO 1)
msg$(1) = msg$

REDIM choice$(-1 TO 0)
IF LEN(yesword$) THEN
     choice$(-1) = yesword$
ELSE
     choice$(-1) = " Yes "
END IF
IF LEN(noword$) THEN
     choice$(0) = noword$
ELSE
     choice$(0) = " No "
END IF

YesNo2 = Dialog(msg$(), choice$(), parm())

ERASE msg$
ERASE choice$

END FUNCTION

