DEFINT A-Z

' $INCLUDE: 'DIR.INC'

DECLARE FUNCTION FileCount (fspec$)
DECLARE FUNCTION SubCount (d$)

FUNCTION FileCount (fspec$)
'***************************************************************************
'FileCount() returns the number of specified files in a directory.  It is
' especially useful when dimensioning an array to hold filenames.
'
'Pass a full pathname followed by the wildcard specifying the files to count.
' If a wildcard is passed without a directory, the current directory will be
' assumed.  A null string defaults to *.* in the current directory.
'
'The function DOES NOT include subdirectories or the parent directories in
' the count.
'
'****************************************************************************

DIM DirInfo AS DirType                  'To capture directory info.

x$ = fspec$                             'Use a temp var.

IF LEN(x$) = 0 THEN x$ = "*.*"          'Default to all files in current dir.

x$ = Dir$(x$, DirInfo)                  'Get the first entry.  We don't need
                                        'x$ anymore.
DO WHILE DirInfo.ErrorCode = 0
     IF NOT DirInfo.DirectoryFlag THEN c = c + 1       'Count those that are
     x$ = Dir$("", DirInfo)                            'not directories.
LOOP

FileCount = c

END FUNCTION

FUNCTION SubCount (d$)
'****************************************************************************
'SubCount() merely returns the number of subdirectories in a directory.  It
' is handy when dimensioning an array.
'
'Pass the name of the directory, or a null string to process the current
' directory.  The specified directory may be passed with or without a
' trailing backslash.
'
'The function only counts subdirectories, not the . or .. parent directories.
'
'****************************************************************************

DIM DirInfo AS DirType                  'To capture directory info.

x$ = d$                                 'Use a temp var for the directory.

IF LEN(x$) THEN                         'Add a trailing backslash if needed.

     IF RIGHT$(x$, 1) <> "\" THEN x$ = x$ + "\"

END IF

x$ = Dir$((x$ + "*.*"), DirInfo)        'Get the first entry.  We don't need
                                        'x$ anymore.
DO WHILE DirInfo.ErrorCode = 0
     IF DirInfo.DirectoryFlag THEN
          IF ASC(x$) <> 46 THEN c = c + 1        'Don't count . and ..
     END IF
     x$ = Dir$("", DirInfo)
LOOP

SubCount = c

END FUNCTION

