DEFINT A-Z

' $INCLUDE: 'FILEPART.INC'

'(Declaration in include file)


'External procedures:

DECLARE FUNCTION Rinstr (start, search$, lookfor$)

FUNCTION FileParts$ (filespec$, part)
'****************************************************************************
'Returns a specified part of an extended filename or other filespec type
' string.
'
'The return value depends upon the value of the part argument, which should
' be one of the constants in the FILEPART.INC file:
'
'Example: filespec$ = "C:\GAMES\SAVEGAME.001"
'         FileParts$(filespec$, FPPATH)       -->  "C:\GAMES\"
'         FileParts$(filespec$, FPFILENAME)   -->  "SAVEGAME.001"
'         FileParts$(filespec$, FPEXTENSION)  -->  "001"
'         FileParts$(filespec$, FPNAMEONLY)   -->  "SAVEGAME"
'         FileParts$(filespec$, FPDRIVE)      -->  "C:"
'
'         filespec$ = "HOMEWORK.TXT"
'         FileParts$(filespec$, FPPATH)  --> "" Returns null if the requested
'         FileParts$(filespec$, FPDRIVE) --> "" info is not part of filespec$
'
'Paths are returned with a trailing backslash.  Drive letters are returned
' with a trailing colon.  Extensions are returned without a leading period.
'
'Quirks:  FileParts$() assumes that all pathnames end in a backslash.  If you
' pass the function one that does not, it will think that it is a filename:
'
' Example: "C:\GAMES" will be interpreted as a file called "GAMES" in the
'           root directory of C:.  "C:\GAMES\" would be Ok.
'
'****************************************************************************

bs$ = "\"                               'For optimization.
p$ = ""                                 'Initialize to null.

fspec$ = LTRIM$(RTRIM$(filespec$))      'Trim any spaces off filespec$

IF fspec$ = p$ THEN                     'If the filespec is a null string,
     FileParts$ = p$                    'return a null string.
     EXIT FUNCTION
END IF

dot = INSTR(fspec$, ".")                'Find a dot, if any.
colon = INSTR(fspec$, ":")              'Find a colon, if any.
lastslash = Rinstr(0, fspec$, bs$)      'Find the last backslash, if any.

SELECT CASE part
     CASE FPFILENAME
          IF lastslash THEN             'Find the left boundary
               l = lastslash + 1
          ELSEIF colon THEN
               l = colon + 1
          ELSE
               l = 1          'It must be a filename only.
          END IF
          p$ = MID$(fspec$, l)
     CASE FPNAMEONLY
          IF lastslash THEN             'Find the left boundary
               l = lastslash + 1
          ELSEIF colon THEN
               l = colon + 1
          ELSE
               l = 1
          END IF
          IF dot THEN                   'Find the right boundary
               r = dot - 1
               p$ = MID$(fspec$, l, (dot - l))
          ELSE
               p$ = MID$(fspec$, l)
          END IF
     CASE FPEXTENSION
          IF dot THEN
               p$ = MID$(fspec$, (dot + 1))
          END IF
     CASE FPPATH
          IF lastslash THEN                      'Regular pathname
               p$ = LEFT$(fspec$, lastslash)
          ELSEIF colon THEN                      'Drive letter only
               p$ = LEFT$(fspec$, colon) + bs$
          END IF
     CASE FPDRIVE
          IF colon THEN
               p$ = LEFT$(fspec$, colon)
          END IF
     CASE ELSE                                                   'What???
          'p$ is already set to null
END SELECT

FileParts$ = p$                         'Return the requested information.

END FUNCTION

