DEFINT A-Z

' $INCLUDE: 'KEYCODE.INC'
' $INCLUDE: 'PARM.INC'
' $INCLUDE: 'SETCURS.INC'
' $INCLUDE: 'TRUEFALS.INC'

DECLARE FUNCTION GenMen (choice$(), ok(), parm())
DECLARE FUNCTION GenMen2 (choice$(), parm())

'External procedures:

DECLARE SUB Center (row, text$)
DECLARE FUNCTION GetKey$ (parm())
DECLARE FUNCTION Istr$ (i)

FUNCTION GenMen (choice$(), ok(), parm())
'****************************************************************************
'GenMen() is a general vertical lightbar menu function.  It will return the
' element number of the selected item or zero if the user presses ESC.
'
'The ok() array is used to specify which choices are available:
'
'                      0=Not available  Non-zero=Ok
'
'Unavailable elements will appear in the FGD/BGN or FGDS/BGS colors depending
' on whether they are highlighted or not.  The ok() array must have
' subscripts equal to those of choice$() or those specified by parm(6 and 7)
' (See below).
'
'    parm(1)  =  top row
'    parm(2)  =  left column  0=Center
'    parm(3)  =  # blank lines between choices  >=0
'    parm(4)  =  allow letter keys?  0=No  Non-zero=Yes
'    parm(5)  =  initial selected choice
'    parm(6)  =  minimum choice$() subscript
'    parm(7)  =  maximum choice$() subscript
'
'The user can press the first letter of each choice to make their selection.
' If a choice's first letter is already taken, the second letter will be
' used.  If the second letter is already in use, the third letter of the
' choice will be used, and so on.  This ability can be turned on or off with
' parm(4).  The letter keys are NOT case sensitive.
'
'If a combination of any of the above parameters cause one or more menu items
' to be placed outside the actual screen area, a run-time error will occur.
'
'parm(6 and 7) can specify minimum and maximum elements of the array to use
' if the actual array contains more elements than you want on the menu.  If
' both parameters are zero, the actual LBOUND/UBOUND values will be used.
'
'    Example:  DIM choice$(-10 to 30)              This example would create
'              (assign values to choice$()...)     a lightbar menu using only
'              parm(6) = 1                         choices 1 through 5,
'              parm(7) = 5                         ignoring any element below
'              picked = GenMen(...)                1 or over 5.
'
'Note: It is not recommended to include subscript zero in the choices sent to
' GenMen().  You will be unable to tell the difference between the user
' selecting element zero and the user pressing ESC.  Exception: When element
' zero is some sort of quit or exit option this might be acceptable.
'
'****************************************************************************

min = LBOUND(choice$)                   'Get information about choice$().
max = UBOUND(choice$)
IF parm(6) OR parm(7) THEN              'Either one non-zero?
     min = parm(6)
     max = parm(7)
END IF

IF parm(4) THEN                         'Set up the letter keys if wanted.
     FOR x = min TO max
          c$ = UCASE$(LTRIM$(RTRIM$(choice$(x))))
          l = 1
          DO
               k$ = MID$(c$, l, 1)
               l = l + 1
          LOOP UNTIL INSTR(ltrs$, k$) = 0 OR l > LEN(c$)
          ltrs$ = ltrs$ + k$
     NEXT x
END IF

sel = parm(5)                           'Determine initial selection
IF sel < min THEN sel = min
IF sel > max THEN sel = max

oldcursor = SetCursor(SCNONE)           'Turn off the cursor

DO

     row = parm(1)                      'Show the menu options
     FOR x = min TO max
          IF ok(x) = 0 THEN COLOR parm(FGD)
          IF x = sel THEN
               COLOR parm(FGS), parm(BGS)
               IF ok(x) = 0 THEN COLOR parm(FGDS)
          END IF
          IF parm(2) = 0 THEN
               Center row, choice$(x)
          ELSE
               LOCATE row, parm(2): PRINT choice$(x);
          END IF
          COLOR parm(FGN), parm(BGN)
          row = row + 1 + parm(3)
     NEXT x

     k$ = UCASE$(GetKey$(parm()))       'Get keyboard input
    
     SELECT CASE KeyCode(k$)
          CASE KCESC                                        'ESC
               GenMen = 0
               EXIT DO
          CASE KCENTER                                      'Enter
               IF ok(sel) THEN
                    GenMen = sel
                    EXIT DO
               END IF
          CASE KCUP                                         'Up arrow
               sel = sel - 1
          CASE KCDOWN                                       'Down arrow
               sel = sel + 1
          CASE ELSE                                         'Letter key?
               IF parm(4) THEN
                    x = INSTR(ltrs$, k$)
                    IF x THEN
                         sel = min + x - 1
                         IF ok(sel) THEN
                              GenMen = sel
                              EXIT DO
                         END IF
                    END IF
               END IF
     END SELECT

     IF sel < min THEN sel = max
     IF sel > max THEN sel = min

LOOP

x = SetCursor(oldcursor)                'Restore the cursor

END FUNCTION

FUNCTION GenMen2 (choice$(), parm())
'****************************************************************************
'GenMen2() is identical to GenMen() except that you need not pass the ok()
' array.  All elements default to available.
'
'See GenMen() for more information.  The parm() settings are identical.
'
'****************************************************************************

min = LBOUND(choice$)
max = UBOUND(choice$)
REDIM ok(min TO max)                    'Create an ok() array and make all
FOR x = min TO max                      'its elements non-zero.
     ok(x) = TRUE
NEXT x

GenMen2 = GenMen(choice$(), ok(), parm())

ERASE ok

END FUNCTION

