DEFINT A-Z

' $INCLUDE: 'KEYCODE.INC'
' $INCLUDE: 'PARM.INC'
' $INCLUDE: 'SAVEENV.INC'
' $INCLUDE: 'SETCURS.INC'

DECLARE FUNCTION ListBox (title$, choice$(), parm())

'External procedures:

DECLARE SUB Box (t, l, b, r, b$)
DECLARE SUB BoxCalc (t, l, b, r, tall, wide)
DECLARE FUNCTION GetKey$ (parm())
DECLARE FUNCTION PadR$ (t$, l)
DECLARE FUNCTION PadX$ (t$, l, s, c$)
DECLARE FUNCTION Istr$ (i)
DECLARE FUNCTION VPage (p)

FUNCTION ListBox (title$, choice$(), parm())
'****************************************************************************
'ListBox() works like PickOne(), but it appears in a pop-up box.  It returns
' the element number of the item selected, or LBOUND(choice$)-1 if the user
' pressed ESC.  There are no hotkeys in ListBox(), but you can easily add
' that functionality.
'
'The title$ argument will be centered on the top border of the box.  If no
' title is desired, pass a null string.
'
'The width of the box is determined by the longer of the title or longest
' choice$() element.
'
'    parm(1) = top row  0=Center
'    parm(2) = left column  0=Center
'    parm(3) = box border type 1-4.  See SUB Panes() for numeric boxtypes.
'    parm(4) = initial selected choice
'    parm(5) = minimum choice$() element to use
'    parm(6) = maximum choice$() element to use
'    parm(7) = allow letter keys?  0=No  <0=Instant select  >0=Alpha Search
'
'If both parm(5 and 6) are zero, the actual LBOUND/UBOUND values of choice$()
' will be used.  If a minimum element is specified, parm(5)-1 will be
' returned if the user presses ESC (You can also check LastKey$() instead of
' relying on the return value).
'
'If parm(7) is zero, pressing anything other than the navigation keys will do
' nothing.  If it is less than zero, pressing the first letter of the item
' will instantly select the item.  See Dialog() for more info.  If the value
' of parm(7) is greater than zero, pressing letter keys will result in an
' alpha search identical to those of PickOne() & PickSome().
'
'****************************************************************************

'                     *** Preliminary calculations ***

IF parm(5) OR parm(6) THEN              'Get information about choice$().
     min = parm(5)
     max = parm(6)
ELSE
     min = LBOUND(choice$)
     max = UBOUND(choice$)
END IF

wide = LEN(title$)                      'Find out how wide & tall to make the
IF wide > 0 THEN wide = wide + 2        'box.  Allow for the title's frame.
FOR x = min TO max
     tall = tall + 1
     l = LEN(choice$(x))
     IF l > wide THEN wide = l
NEXT x
IF tall > 10 THEN tall = 10

row1 = parm(1)                          'Find out where to place the box.
col1 = parm(2)
BoxCalc row1, col1, row2, col2, tall, wide

IF parm(7) < 0 THEN                     'Set up the letter keys if wanted.
     FOR x = min TO max
          c$ = UCASE$(LTRIM$(RTRIM$(choice$(x))))
          l = 1
          DO
               l$ = MID$(c$, l, 1)
               l = l + 1
          LOOP UNTIL INSTR(ltrs$, l$) = 0 OR l > LEN(c$)
          ltrs$ = ltrs$ + l$
     NEXT x
END IF

'                          *** Draw the Box ***

DIM OldEnv AS EnvironType               'Save/Change the environment.
SaveEnv OldEnv
x = SetCursor(SCNONE)                   'Turn off the cursor

workpage = VPage(0)                     'Allocate a non-critical video page.
PCOPY 0, workpage                       'Copy the screen to the scratch page.
SCREEN , , workpage, 0                  'Draw on the work page until ready.

COLOR parm(FGWB), parm(BGWB)            'Draw the outline & title
Box row1, col1, row2, col2, Istr$(parm(3))
SELECT CASE parm(3)
     CASE 2
          lc$ = CHR$(181)
          rc$ = CHR$(198)
          h$ = CHR$(205)
          v$ = CHR$(186)
     CASE 3
          lc$ = CHR$(181)
          rc$ = CHR$(198)
          h$ = CHR$(205)
          v$ = CHR$(179)
     CASE 4
          lc$ = CHR$(180)
          rc$ = CHR$(195)
          h$ = CHR$(198)
          v$ = CHR$(186)
     CASE ELSE
          lc$ = CHR$(180)
          rc$ = CHR$(195)
          h$ = CHR$(198)
          v$ = CHR$(179)
END SELECT
IF LEN(title$) THEN
     x = wide - (LEN(title$) + 2)
     LOCATE row1, col1 + (x \ 2) + 1
     PRINT lc$; title$; rc$;
END IF

COLOR 0, 0                              'Print the shadow
l = col2 + 1
FOR x = (row1 + 1) TO row2
     LOCATE x, l: PRINT " "
NEXT x
LOCATE row2 + 1, col1 + 1: PRINT SPACE$(wide + 2);

PCOPY workpage, 0                       'Pop the box onto the screen.
SCREEN , , 0, 0                         'Draw on screen 0 again.
x = VPage(workpage)                     'Release the scratch video page.

'                    *** Pick a choice, any choice! ***

sel = parm(4)                           'Initially position the list.
IF sel < min OR sel > max THEN sel = min
top = min
bot = top + tall - 1
DO WHILE bot < sel
     top = top + 1
     bot = bot + 1
LOOP
col = col1 + 1

id$ = CHR$(18)                          'The little indicator character.
irow = row1 + 1

COLOR parm(FGWT), parm(BGWT)

DO                                      'The main loop!
    
     row = row1                         'Print the choices.
     FOR x = top TO bot
          row = row + 1
          IF x = sel THEN COLOR parm(FGWS), parm(BGWS)
          LOCATE row, col
          PRINT PadR$(choice$(x), wide)
          COLOR parm(FGWT), parm(BGWT)
     NEXT x

     IF tall = 10 THEN                  'Put an indicator on the side.
          COLOR parm(FGWB), parm(BGWB)
          LOCATE irow, col2
          PRINT v$;
          x = INT((sel / max) * 10)
          IF x < 1 THEN x = 1
          irow = row1 + x
          LOCATE irow, col2
          PRINT id$;
          COLOR parm(FGWT), parm(BGWT)
     END IF

     IF parm(7) > 0 THEN                'Show the search string on the bottom
          COLOR parm(FGWB), parm(BGWB)
          LOCATE row2, col1 + 1
          PRINT PadX$(search$, wide, 0, h$);
          COLOR parm(FGWT), parm(BGWT)
     END IF

     k$ = UCASE$(GetKey$(parm()))            'Get keyboard input
     k = KeyCode(k$)

     SELECT CASE k
          CASE KCESC                         'ESC
               sel = min - 1
               EXIT DO
          CASE KCENTER                       'Enter
               EXIT DO
          CASE KCUP                          'Up arrow
               sel = sel - 1
               IF sel < min THEN sel = min
               IF sel < top THEN
                    top = top - 1
                    bot = bot - 1
               END IF
          CASE KCDOWN                        'Down arrow
               sel = sel + 1
               IF sel > max THEN sel = max
               IF sel > bot THEN
                    top = top + 1
                    bot = bot + 1
               END IF
          CASE KCPGUP                        'PgUp
               IF top > min THEN
                    top = top - tall
                    bot = bot - tall
                    IF top < min THEN
                         top = min
                         bot = top + tall - 1
                    END IF
                    IF sel > bot THEN sel = bot
               END IF
          CASE KCPGDN                        'PgDn
               IF bot < max THEN
                    top = top + tall
                    bot = bot + tall
                    IF bot > max THEN
                         bot = max
                         top = bot - tall + 1
                    END IF
                    IF sel < top THEN sel = top
               END IF
          CASE KCHOME                        'Home
               sel = min
               top = min
               bot = top + tall - 1
          CASE KCEND                         'End
               sel = max
               bot = max
               top = bot - tall + 1
          CASE KCBACKSPACE
               l = LEN(search$)
               IF l THEN search$ = LEFT$(search$, l - 1)
          CASE 32 TO 126
               IF parm(7) < 0 THEN           'Instant select
                    l = INSTR(ltrs$, k$)
                    IF l THEN
                         sel = min + l - 1
                         EXIT DO
                    END IF
               ELSEIF parm(7) > 0 THEN       'Alpha search
                    l = LEN(search$)
                    IF l <= wide - 2 THEN
                         search$ = search$ + k$
                         l = l + 1
                    END IF
                    found = min - 1
                    FOR x = sel TO max
                         IF LEFT$(UCASE$(choice$(x)), l) = search$ THEN
                              found = x
                              EXIT FOR
                         END IF
                    NEXT x
                    IF found < min THEN
                         FOR x = min TO sel - 1
                              IF LEFT$(UCASE$(choice$(x)), l) = search$ THEN
                                   found = x
                                   EXIT FOR
                              END IF
                         NEXT x
                    END IF
                    IF found >= min THEN
                         sel = found
                         IF sel < top OR sel > bot THEN
                              top = min
                              bot = top + tall - 1
                         END IF
                         DO WHILE sel > bot
                              top = top + tall
                              bot = top + tall - 1
                         LOOP
                         IF bot > max THEN
                              bot = max
                              top = bot - tall + 1
                         END IF
                    END IF
               END IF
          CASE ELSE
               'Do nothing
     END SELECT

     IF k <> KCBACKSPACE AND (k < 32 OR k > 126) THEN search$ = ""

LOOP

'                     *** Clean up after ourselves ***

RestEnv OldEnv

ListBox = sel                           'Return highlighted element

END FUNCTION

