DEFINT A-Z

' $INCLUDE: 'KEYCODE.INC'
' $INCLUDE: 'PARM.INC'
' $INCLUDE: 'SETCURS.INC'
' $INCLUDE: 'TRUEFALS.INC'

DECLARE FUNCTION MsgMenu (choice$(), msg$(), ok(), parm())
DECLARE FUNCTION MsgMenu2 (choice$(), msg$(), parm())

'External procedures:

DECLARE SUB Center (row, text$)
DECLARE FUNCTION GetKey$ (parm())
DECLARE FUNCTION Istr$ (i)
DECLARE FUNCTION PadC$ (t$, l)
DECLARE FUNCTION PadL$ (t$, l)
DECLARE FUNCTION PadR$ (t$, l)

FUNCTION MsgMenu (choice$(), msg$(), ok(), parm())
'****************************************************************************
'MsgMenu() is a general vertical lightbar menu function.  Along with the menu
' options, it displays another lengthier message on a specified row.  It will
' return the element number of the selected item or LBOUND()-1 if the user
' presses ESC.
'
'The ok() array is used to specify which choices are available:
'
'                      0=Not available  Non-zero=Ok
'
'The msg$() and ok() arrays must have subscripts equal to those of choice$(),
' or those specified by parm(6 and 7) - See below.
'
'    parm(1)  =  top row
'    parm(2)  =  left column  0=Center
'    parm(3)  =  # blank lines between choices  >=0
'    parm(4)  =  allow letter keys?  0=No  Non-zero=Yes
'    parm(5)  =  initial selected choice
'    parm(6)  =  minimum choice$() subscript
'    parm(7)  =  maximum choice$() subscript
'    parm(8)  =  message row  Default=24
'    parm(9)  =  message justification  <0=Left  0=Center  >0=Right
'    parm(10) =  message color  0=normal color  Non-zero=highlighted color
'
'The user can press the first letter of each choice to make their selection.
' If a choice's first letter is already taken, the second letter will be
' used.  If the second letter is already in use, the third letter of the
' choice will be used, and so on.  This ability can be turned on or off with
' parm(4).  The letter keys are NOT case sensitive.  The choice$() array must
' start with subscript one (1) for this to work properly.
'
'If both parm(6) and parm(7) are zero, LBOUND() & UBOUND() will be assumed.
' If parm(6) is specified and ESC is pressed, parm(6)-1 will be returned.
' You can also check LastKey$().
'
'If a combination of any of the above parameters cause one or more menu items
' to be placed outside the actual screen area, a run-time error will occur.
'
'For more information, see the functions GenMen() and GenMen2().
'
'****************************************************************************

IF parm(6) OR parm(7) THEN              'Determine which elements to use.
     min = parm(6)
     max = parm(7)
ELSE
     min = LBOUND(choice$)
     max = UBOUND(choice$)
END IF

IF parm(4) THEN                         'Set up the letter keys if wanted.
     FOR x = min TO max
          c$ = UCASE$(LTRIM$(RTRIM$(choice$(x))))
          l = 1
          DO
               k$ = MID$(c$, l, 1)
               l = l + 1
          LOOP UNTIL INSTR(ltrs$, k$) = 0 OR l > LEN(c$)
          ltrs$ = ltrs$ + k$
     NEXT x
END IF

sel = parm(5)                           'Determine initial selection.
IF sel < min THEN sel = min
IF sel > max THEN sel = max

msgrow = parm(8)                        'Message row
IF msgrow < 1 THEN msgrow = 24

oldcursor = SetCursor(SCNONE)           'Turn off the cursor.

DO                                      'The main loop:

     row = parm(1)                      'Show the menu options.
     FOR x = min TO max
          IF ok(x) = 0 THEN COLOR parm(FGD)
          IF x = sel THEN
               COLOR parm(FGS), parm(BGS)
               IF ok(x) = 0 THEN COLOR parm(FGDS)
          END IF
          IF parm(2) = 0 THEN
               Center row, choice$(x)
          ELSE
               LOCATE row, parm(2): PRINT choice$(x);
          END IF
          COLOR parm(FGN), parm(BGN)
          row = row + 1 + parm(3)
     NEXT x

     LOCATE msgrow, 1                   'Show the current message.
     IF parm(10) THEN COLOR parm(FGH)
     SELECT CASE parm(9)
          CASE IS < 0
               PRINT PadR$(msg$(sel), 80);
          CASE IS > 0
               PRINT PadL$(msg$(sel), 80);
          CASE ELSE
               PRINT PadC$(msg$(sel), 80);
     END SELECT
     COLOR parm(FGN)

     k$ = UCASE$(GetKey$(parm()))       'Get keyboard input:
    
     SELECT CASE KeyCode(k$)
          CASE KCESC                                        'ESC
               MsgMenu = min - 1
               EXIT DO
          CASE KCENTER                                      'Enter
               IF ok(sel) THEN
                    MsgMenu = sel
                    EXIT DO
               END IF
          CASE KCUP                                         'Up arrow
               sel = sel - 1
          CASE KCDOWN                                       'Down arrow
               sel = sel + 1
          CASE ELSE                                         'Letter key?
               IF parm(4) THEN
                    x = INSTR(ltrs$, k$)
                    IF x THEN
                         sel = min + x - 1
                         IF ok(sel) THEN
                              MsgMenu = sel
                              EXIT DO
                         END IF
                    END IF
               END IF
     END SELECT

     IF sel < min THEN sel = max
     IF sel > max THEN sel = min

LOOP

x = SetCursor(oldcursor)                'Restore the cursor

END FUNCTION

FUNCTION MsgMenu2 (choice$(), msg$(), parm())
'****************************************************************************
'MsgMenu2() is identical to MsgMenu() except that you need not pass the ok()
' array.  All elements default to available.
'
'See MsgMenu() for more information.  The parm() settings are identical.
'
'****************************************************************************

IF parm(6) OR parm(7) THEN              'Determine how big to make ok().
     min = parm(6)
     max = parm(7)
ELSE
     min = LBOUND(choice$)
     max = UBOUND(choice$)
END IF

REDIM ok(min TO max)                    'Create the ok() array and make all
FOR x = min TO max                      'its elements non-zero.
     ok(x) = TRUE
NEXT x

MsgMenu2 = MsgMenu(choice$(), msg$(), ok(), parm())

ERASE ok

END FUNCTION

