DEFINT A-Z

DECLARE FUNCTION PadC$ (orig$, newlen)
DECLARE FUNCTION PadL$ (orig$, newlen)
DECLARE FUNCTION PadR$ (orig$, newlen)
DECLARE FUNCTION PadX$ (orig$, newlen, side, char$)

FUNCTION PadC$ (orig$, newlen)

PadC$ = PadX$(orig$, newlen, 0, " ")

END FUNCTION

FUNCTION PadL$ (orig$, newlen)

PadL$ = PadX$(orig$, newlen, -1, " ")

END FUNCTION

FUNCTION PadR$ (orig$, newlen)

PadR$ = PadX$(orig$, newlen, 1, " ")

END FUNCTION

FUNCTION PadX$ (orig$, newlen, side, char$)
'****************************************************************************
'Pads a string with a specified character on the specified side(s) up to the
' new length.  A more flexible version of the other "Pad" functions (all of
' which are merely translated into calls to PadX$() with a space for the pad
' character!).
'
'If the original is longer than the new length, it will be TRUNCATED to this
' length.  If center-padding, characters will be taken equally off each side
' of the original.
'
'The side argument is expressed like so:  <0 = Left
'                                          0 = Center
'                                         >0 = Right
'
'Examples: PadX$("Hello!",10,1," ") --> "Hello!    "
'          PadX$("$500",10,-1,"*")  --> "******$500"
'          PadX$("WOW",20,0,"!")    --> "!!!!!!!!WOW!!!!!!!!!"
'          PadX$("12345",4,-1,"0")  --> "2345"
'
'****************************************************************************

IF char$ = "" THEN                 'If char$ is null, default to a space.
     c$ = " "
ELSE
     c$ = char$
END IF

l = LEN(orig$)                     'Find length of original.

IF l = newlen THEN                 'If already equal to new length, return
     PadX$ = orig$                 'the original.
     EXIT FUNCTION
END IF

pad$ = STRING$(newlen, c$)         'Make a string completely of pad chars.

SELECT CASE side
     CASE IS < 0                                       'Pad Left
          PadX$ = RIGHT$(pad$ + orig$, newlen)
     CASE IS > 0                                       'Pad Right
          PadX$ = LEFT$(orig$ + pad$, newlen)
     CASE ELSE                                         'Pad Center
          IF l > newlen THEN                                'Original > New
               diff = l - newlen
               diff = diff \ 2 + 1                'Take the middle chunk
               PadX$ = MID$(orig$, diff, newlen)
          ELSE                                              'New > Original
               diff = newlen - l
               l = diff \ 2                       'Add padding to each side
               r = diff - l
               PadX$ = STRING$(l, c$) + orig$ + STRING$(r, c$)
          END IF
END SELECT

END FUNCTION

