DEFINT A-Z

' $INCLUDE: 'TRUEFALS.INC'

DECLARE SUB Box (t, l, b, r, boxtype$)
DECLARE SUB Panes (t, l, b, r, mr, mc, boxtype$)


'External Procedures:

DECLARE FUNCTION IsAlpha (s$)

SUB Box (t, l, b, r, boxtype$)
'****************************************************************************
'Draws a box on the screen at the specified coordinates.  t, l, b and r are
' the top, left, bottom and right coordinates respectively.  The appearance
' of the box is determined by boxtype$, which can either be one character to
' pick a predefined box type or a string of 11 or more characters to be used
' as the actual box characters.  See the function body of Panes() for the
' exact placement of the characters within the string and other options.
'
'Examples of boxtype$: "1" or "S"    Draws a single-line box (default)
'                      "2" or "D"    Draws a double-line box
'                      "3" or "H"    Double Horizontal lines, single vertical
'                      "4" or "V"    Double Vertical lines, single horizontal
'                      "***********" Draws a box made of asterisks
'
'****************************************************************************

'Translate the call to Box() into a call to Panes() with no inner lines.

Panes t, l, b, r, 0, 0, boxtype$

END SUB

SUB Panes (t, l, b, r, mr, mc, boxtype$)
'****************************************************************************
'Draws a box on the screen at the specified coordinates.  t, l, b and r are
' the top, left, bottom and right box coordinates respectively.  mr and mc
' are the middle row and middle column, specifying where the box is to be
' split horizontally and/or vertically.  If either or both mr or mc are zero,
' the box will not be split in that direction.  Experiment with it.
'
'The appearance of the box is determined by boxtype$, which can either be one
' character to pick a predefined box type or a string of 11 or more to be
' used as the actual box characters.  See the function body for the exact
' placement of the characters within the string.
'
'Examples of boxtype$: "1" or "S"    Draws a single-line box (default)
'                      "2" or "D"    Draws a double-line box
'                      "3" or "H"    Double horizontal lines, single vertical
'                      "4" or "V"    Double vertical lines, single horizontal
'                      "***********" Draws a box made of asterisks
'
'The box can be drawn as an outline only, not overwriting anything within the
' box's borders or can be filled with a fill character, effectively placing
' the box over whatever was already there.  This option is also controlled by
' the boxtype$ argument:
'
'If boxtype$ is specified as a number ("1", "2"...) the box will be drawn as
' an outline only.  If boxtype$ is specified as a letter ("S", "D"...) the
' box will be filled with spaces.
'
'If boxtype is a user-supplied string of characters, if it's length is 12 or
' more, the 12th character will be used as the fill character, otherwise the
' box will be drawn as an outline.
'
'****************************************************************************

'                     *** Define the box characters ***

x$ = "ڿ´"                 'Default the box characters to single line

IF LEN(boxtype$) >= 11 THEN                                 'User-supplied
     x$ = boxtype$                                          'box characters
ELSE
     SELECT CASE UCASE$(LEFT$(boxtype$, 1))
          CASE "2", "D"                                     'Double
               x$ = "ɻȺ˹"
          CASE "3", "H"                                     'Horizontal dbl.
               x$ = "ոԳѵ"
          CASE "4", "V"                                     'Vertical dbl.
               x$ = "ַӺҶ"
          CASE ELSE                                         'Single (default)
               'Use the pre-defined value of x$
     END SELECT
     IF IsAlpha(boxtype$) THEN                              'Fill the box if
          x$ = x$ + " "                                     'boxtype$ is a
     END IF                                                 'letter.
END IF

'           *** Give descriptive names to the box characters ***

tlc$ = MID$(x$, 1, 1)                        'Top Left Corner
trc$ = MID$(x$, 2, 1)                        'Top Right Corner
brc$ = MID$(x$, 3, 1)                        'Bottom Right Corner
blc$ = MID$(x$, 4, 1)                        'Bottom Left Corner
ver$ = MID$(x$, 5, 1)                        'Vertical line
hor$ = MID$(x$, 6, 1)                        'Horizontal line
tx$ = MID$(x$, 7, 1)                         'Top intersection
rx$ = MID$(x$, 8, 1)                         'Right intersection
bx$ = MID$(x$, 9, 1)                         'Bottom intersection
lx$ = MID$(x$, 10, 1)                        'Left intersection
mx$ = MID$(x$, 11, 1)                        'Middle intersection

'                           *** Draw the Box ***

wide = r - l - 1
across$ = STRING$(wide, hor$)
IF LEN(x$) > 11 THEN                                   'Fill character?
     f$ = STRING$(wide, MID$(x$, 12, 1))
     fill = TRUE
END IF

LOCATE t, l                                            'The main box outline:
PRINT tlc$; across$; trc$;                             'The top line
FOR x = (t + 1) TO (b - 1)                             'The sides
     LOCATE x, l
     PRINT ver$;
     IF fill THEN                                      'If the box is being
          PRINT f$; ver$;                              'filled, put the fill
     ELSE                                              'characters between
          LOCATE x, r                                  'the sides. Otherwise
          PRINT ver$;                                  'just skip that area.
     END IF
NEXT x
LOCATE b, l                                            'The bottom line
PRINT blc$; across$; brc$;

IF mr > 0 THEN                                         'Split horizontally?
     LOCATE mr, l
     PRINT lx$; across$; rx$;
END IF

IF mc > 0 THEN                                         'Split vertically?
     LOCATE t, mc
     PRINT tx$;
     FOR x = (t + 1) TO (b - 1)
          LOCATE x, mc
          PRINT ver$;
     NEXT x
     LOCATE b, mc
     PRINT bx$;
END IF

IF mr > 0 AND mc > 0 THEN                              'Split both ways?
     LOCATE mr, mc
     PRINT mx$
END IF

END SUB

