DEFINT A-Z

' $INCLUDE: 'KEYCODE.INC'
' $INCLUDE: 'PARM.INC'
' $INCLUDE: 'SETCURS.INC'
' $INCLUDE: 'TRUEFALS.INC'

DECLARE FUNCTION PickOne (choice$(), parm())

'External procedures:

DECLARE SUB BorderLines (parm())
DECLARE SUB Center (row, txt$)
DECLARE FUNCTION GetKey$ (parm())
DECLARE FUNCTION PadR$ (orig$, newlen)
DECLARE SUB WipeArea (t, l, b, r)

FUNCTION PickOne (choice$(), parm())
'****************************************************************************
'Allow the user to select an item from an array by highlighting it with the
' cursor keys & pressing Enter.  The function returns the element number of
' the currently highlighted item.  Check the LastKey$() function to see what
' key was used to exit the function.
'
'    parm(1) = top row
'    parm(2) = bottom row
'    parm(3) = column width  0=Calculated by the function (recommended)
'    parm(4) = initial selected element #
'    parm(5) = reset?  0=Subsequent call  Non-zero=Reset (First time call)
'    parm(6) = minimum element to use
'    parm(7) = maximum element to use
'    parm(8) = allow alpha search?  0=No  Non-zero=Yes
'    parm(9) = Additonal hotkey(s)?  0=No  Non-zero=Yes  (See below)
'
'Any column width specified in parm(3) will be increased by 2 to allow for
' spaces on either side of each item.  Allow for this when supplying this
' value.
'
'If both parm(6) and parm(7) are zero, the LBOUND()/UBOUND() values will be
' used.
'
'If parm(8) is non-zero, the user can seek a value by typing in the first few
' letters of the item.  Pressing any of the movement keys (arrow, PgUp, etc.)
' will clear the current search string.  The backspace key can be used to
' delete the last character of the search string.  The search is not case-
' sensitive.
'
'parm(9 to MINPARM-1) hold key codes that act as exit keys in addition to
' Enter and ESC.  See function KeyCode() for the kind of values expected in
' these elements.  If no hotkeys are desired, make parm(9) equal to zero.
' The function only checks the array until it encounters a zero.  Check the
' LastKey$() function to see which of your hotkeys (if any) was used to exit
' the function.
'
'When returning to the function after processing an (alternate) exit key,
' make sure that parm(4) is updated to reflect the current element, and
' parm(5) is zero.  If calling the function for the first time, make sure
' parm(5) is non-zero.
'
'See also: KeyCode(), LastKey$()
'
'****************************************************************************

STATIC top                              'To preserve the position of the pick
                                        'screen between calls.

oldcursor = SetCursor(SCNONE)           'Turn the cursor off

IF parm(6) OR parm(7) THEN              'Determine which elements to use.
     min = parm(6)
     max = parm(7)
ELSE
     min = LBOUND(choice$)
     max = UBOUND(choice$)
END IF

REDIM temp(1 TO MAXPARM)                'Create a duplicate parameter array
FOR x = MINPARM TO MAXPARM              'for calling the BorderLines() SUB.
     temp(x) = parm(x)
NEXT x
temp(1) = parm(1)
temp(2) = parm(2)
temp(5) = min
temp(6) = max

wide = parm(3)                          'Calculate column widths, increasing
IF wide < 1 THEN                        ' the given or calculated value by 2
     FOR x = min TO max                 ' to allow for separating spaces.
          l = LEN(choice$(x)) + 2
          IF l > wide THEN wide = l
     NEXT x
     IF wide < 3 THEN wide = 3
ELSE
     wide = wide + 2
END IF
IF wide > 80 THEN wide = 80
DO WHILE (80 MOD wide) > 0              'Make the columns fill the screen.
     wide = wide + 1
LOOP
cols = 80 \ wide                        'Calculate # of columns

tall = parm(2) - parm(1) - 1            'Calculate # of items per column.
ptot = cols * tall                      'Calculate # of items per screen.

sel = parm(4)                           'Determine initial selected element.
IF sel < min THEN sel = min
IF sel > max THEN sel = max

IF parm(5) THEN top = min               'Was top Reset?  Is it valid?
IF top < min OR top > max THEN top = min
bot = top + ptot - 1

IF sel < top OR sel > bot THEN          'Move top & bot to fit sel.
     top = min
     bot = top + ptot - 1
END IF
DO WHILE sel > bot
     top = top + ptot
     bot = top + ptot - 1
LOOP
IF bot > max THEN bot = max

'    *************************  The Main Loop!  *************************

refresh = TRUE                          'Make sure the screen gets drawn!

DO

     l = wide - 2                       'To allow for the separating spaces.
    
     IF refresh THEN                    'This stuff only needs to be printed
          temp(3) = top                 ' occasionally (i.e., when top & bot
          temp(4) = bot                 ' change).
          BorderLines temp()
          IF LEN(search$) THEN
               COLOR parm(FGH)
               Center parm(2), search$
          END IF
          WipeArea parm(1) + 1, 1, parm(2) - 1, 80
          COLOR parm(FGN), parm(BGN)
          refresh = FALSE
     END IF

     row = parm(1) + 1: col = 1         'Show the items on screen
     FOR x = top TO bot
          IF x = sel THEN COLOR parm(FGS), parm(BGS)
          LOCATE row, col: PRINT " "; PadR$(choice$(x), l); " "
          COLOR parm(FGN), parm(BGN)
          row = row + 1
          IF row = parm(2) THEN row = parm(1) + 1: col = col + wide
     NEXT x

     k$ = UCASE$(GetKey$(parm()))
     k = KeyCode(k$)

     SELECT CASE k
          CASE KCENTER, KCESC                               'Enter & ESC
               EXIT DO
          CASE KCUP                                         'Up Arrow
               sel = sel - 1
          CASE KCDOWN                                       'Down Arrow
               sel = sel + 1
          CASE KCLEFT                                       'Left Arrow
               IF cols > 1 THEN
                    sel = sel - tall
                    IF sel < top THEN
                         sel = sel + ptot
                    END IF
               END IF
          CASE KCRIGHT                                      'Right Arrow
               IF cols > 1 THEN
                    sel = sel + tall
                    IF sel > bot THEN
                         sel = sel - ptot
                    END IF
               END IF
          CASE KCPGUP                                       'PgUp
               IF top > min THEN
                    top = top - ptot
                    IF top < min THEN top = min
                    bot = top + ptot - 1
                    IF bot > max THEN bot = max
                    sel = top
                    refresh = TRUE
               END IF
          CASE KCPGDN                                       'PgDn
               IF bot < max THEN
                    top = top + ptot
                    bot = top + ptot - 1
                    IF bot > max THEN bot = max
                    sel = top
                    refresh = TRUE
               END IF
          CASE KCHOME                                       'Home
               sel = min
               IF top > min THEN
                    top = min
                    bot = top + ptot - 1
                    IF bot > max THEN bot = max
                    refresh = TRUE
               END IF
          CASE KCEND                                        'End
               sel = max
               IF bot < max THEN
                    bot = max
                    top = bot - ptot + 1
                    IF top < min THEN top = min
                    refresh = TRUE
               END IF
          CASE ELSE                                         'Hotkey/Search
               FOR x = 9 TO (MINPARM - 1)                   'Check hotkeys
                    IF parm(x) = 0 THEN EXIT FOR
                    IF k = parm(x) THEN EXIT DO
               NEXT x
               IF parm(8) THEN                              'Allow search?
                    l = LEN(search$)
                    IF k = KCBACKSPACE AND l THEN           'Backspace key
                         search$ = LEFT$(search$, l - 1)
                         refresh = TRUE
                    ELSEIF k >= 32 AND k <= 126 THEN        'Regular keys                
                         IF l <= wide - 2 AND l < 60 THEN
                              search$ = search$ + k$
                              l = l + 1
                              refresh = TRUE
                         END IF
                         found = min - 1
                         FOR x = sel TO max
                              IF LEFT$(UCASE$(choice$(x)), l) = search$ THEN
                                   found = x
                                   EXIT FOR
                              END IF
                         NEXT x
                         IF found < min THEN
                              FOR x = min TO sel - 1
                                   IF LEFT$(UCASE$(choice$(x)), l) = search$ THEN
                                        found = x
                                        EXIT FOR
                                   END IF
                              NEXT x
                         END IF
                         IF found >= min THEN
                              sel = found
                              IF sel < top OR sel > bot THEN
                                   top = min
                                   bot = top + ptot - 1
                              END IF
                              DO WHILE sel > bot
                                   top = top + ptot
                                   bot = top + ptot - 1
                              LOOP
                              IF bot > max THEN bot = max
                              refresh = TRUE
                         END IF
                    END IF
               END IF
     END SELECT

     IF sel < top THEN sel = bot
     IF sel > bot THEN sel = top

     IF k <> KCBACKSPACE AND (k < 32 OR k > 126) AND LEN(search$) THEN
          search$ = ""
          refresh = TRUE
     END IF

LOOP

x = SetCursor(oldcursor)                'Restore cursor to previous setting
ERASE temp                              'Relinquish array memory

PickOne = sel                           'Return the current element number.

END FUNCTION

