DEFINT A-Z

' $INCLUDE: 'KEYCODE.INC'
' $INCLUDE: 'PARM.INC'
' $INCLUDE: 'SETCURS.INC'
' $INCLUDE: 'TAGGING.INC'
' $INCLUDE: 'TRUEFALS.INC'

DECLARE FUNCTION PickSome (choice$(), tag(), parm())

'External procedures:

DECLARE SUB BorderLines (parm())
DECLARE SUB Center (row, txt$)
DECLARE FUNCTION GetKey$ (parm())
DECLARE FUNCTION PadR$ (orig$, newlen)
DECLARE SUB WipeArea (t, l, b, r)

FUNCTION PickSome (choice$(), tag(), parm())
'****************************************************************************
'PickSome() works just like the PickOne() function in that it returns the
' subscript of the currently highlighted element.  In addition, the function
' allows for the tagging of multiple items.  See PickOne() for general
' information about the function and the hotkey arguments.  Additional
' information on how the tagging works is described here.
'
'    parm(1) = top row
'    parm(2) = bottom row
'    parm(3) = column width  0=Calculated by the function (recommended)
'    parm(4) = initial selected element #
'    parm(5) = reset  0=Subsequent call  Non-zero=Reset (First-time call)
'    parm(6) = minimum element to use
'    parm(7) = maximum element to use
'    parm(8) = allow alpha search?  0=No  Non-zero=Yes
'    parm(9) = tagging key  Default=KCSPACEBAR
'    parm(10) = tag all key  Default=KCF6
'    parm(11) = tag none key  Default=KCF7
'    parm(12) = toggle tags key  Default=KCF8
'    parm(13) = additional hotkey(s)?  0=No  Non-zero=Yes (See below)
'
'The tagging keys specified by parm(9 to 12) may be disabled by passing -1.
' The default will be assigned if zero is passed.
' The tagging key will toggle an individual item's tag to on (1) or off (0).
' The tag all/tag none keys will set all items' tags to on/off respectively.
' The toggle tags key will do just that: flip all tags.
'
'The tag array must be an integer array with subscripts identical to the
' choice$() array.  You may pre-tag items or disable items in the array by
' setting elements of tag() to one of the TAGGING.INC constants; UNTAGGED,
' TAGGED, or DISABLED.
'
'If an item is disabled, it will be unaffected by any tagging operations and
' will appear in the dimmed color specified by parm(FGD) and/or parm(FGDS).
'
'****************************************************************************

STATIC top                              'To restore for a subsequent call.

oldcursor = SetCursor(SCNONE)           'Turn the cursor off

                                        'Set up the tagging keys:
kTag = parm(9)
IF kTag = 0 THEN kTag = KCSPACEBAR           'Default = spacebar
IF kTag = -1 THEN kTag = 9999                'Disabled?
kAll = parm(10)
IF kAll = 0 THEN kAll = KCF6                 'Default = F6
IF kAll = -1 THEN kAll = 9998
kNone = parm(11)
IF kNone = 0 THEN kNone = KCF7               'Default = F7
IF kNone = -1 THEN kNone = 9997
kToggle = parm(12)
IF kToggle = 0 THEN kToggle = KCF8           'Default = F8
IF kToggle = -1 THEN kToggle = 9996

REDIM t$(DISABLED TO TAGGED)            'Set up the tagging identifiers
t$(DISABLED) = " "
t$(UNTAGGED) = " "
t$(TAGGED) = CHR$(251)                       'A check-mark

IF parm(6) OR parm(7) THEN              'Determine which choice$() elements
     min = parm(6)                      'to use.
     max = parm(7)
ELSE
     min = LBOUND(choice$)
     max = UBOUND(choice$)
END IF

REDIM temp(1 TO MAXPARM)                'Create a duplicate parameter array
FOR x = MINPARM TO MAXPARM              'for calling the BorderLines() SUB.
     temp(x) = parm(x)
NEXT x
temp(1) = parm(1)
temp(2) = parm(2)
temp(5) = min
temp(6) = max

wide = parm(3)                          'Calculate column widths, increasing
IF wide < 1 THEN                        ' the given or calculated value by 2
     FOR x = min TO max                 ' to allow for separating spaces and
          l = LEN(choice$(x)) + 2       ' the tag character.
          IF l > wide THEN wide = l
     NEXT x
     IF wide < 3 THEN wide = 3
ELSE
     wide = wide + 2
END IF
IF wide > 80 THEN wide = 80
DO WHILE (80 MOD wide) > 0              'Make the columns fill the screen.
     wide = wide + 1
LOOP
cols = 80 \ wide                        'Calculate # of columns

tall = parm(2) - parm(1) - 1            'Calculate # of items per column.
ptot = cols * tall                      'Calculate # of items per screen.

sel = parm(4)                           'Determine initial selected element.
IF sel < min THEN sel = min
IF sel > max THEN sel = max


IF parm(5) THEN top = min               'Was top Reset?  Is it valid?
IF top < min OR top > max THEN top = min
bot = top + ptot - 1

IF sel < top OR sel > bot THEN          'Move top & bot to fit sel if they
     top = min                          'don't already do so.
     bot = top + ptot - 1
END IF
DO WHILE sel > bot
     top = top + ptot
     bot = top + ptot - 1
LOOP
IF bot > max THEN bot = max

'    *************************  The Main Loop!  *************************

refresh = TRUE                          'Make sure the screen gets drawn!

DO

     IF refresh THEN                    'This stuff only needs to be printed
          temp(3) = top                 ' occasionally (i.e., when top & bot
          temp(4) = bot                 ' change).
          BorderLines temp()
          IF LEN(search$) THEN
               COLOR parm(FGH)
               Center parm(2), search$
          END IF
          WipeArea parm(1) + 1, 1, parm(2) - 1, 80
          COLOR parm(FGN), parm(BGN)
          refresh = FALSE
     END IF

     row = parm(1) + 1                  'Show the items on screen
     col = 1
     l = wide - 2
     FOR x = top TO bot
          IF tag(x) = DISABLED THEN COLOR parm(FGD)
          IF x = sel THEN
               COLOR parm(FGS), parm(BGS)
               IF tag(x) = DISABLED THEN COLOR parm(FGDS)
          END IF
          LOCATE row, col: PRINT " "; PadR$(choice$(x), l); t$(tag(x))
          COLOR parm(FGN), parm(BGN)
          row = row + 1
          IF row = parm(2) THEN
               row = parm(1) + 1
               col = col + wide
          END IF
     NEXT x
     
     k$ = UCASE$(GetKey$(parm()))       'Get keyboard input
     k = KeyCode(k$)

     SELECT CASE k
          CASE KCENTER, KCESC                               'Enter & ESC
               EXIT DO
          CASE KCUP                                         'Up Arrow
               sel = sel - 1
          CASE KCDOWN                                       'Down Arrow
               sel = sel + 1
          CASE KCLEFT                                       'Left Arrow
               IF cols > 1 THEN
                    sel = sel - tall
                    IF sel < top THEN
                         sel = sel + ptot
                    END IF
               END IF
          CASE KCRIGHT                                      'Right Arrow
               IF cols > 1 THEN
                    sel = sel + tall
                    IF sel > bot THEN
                         sel = sel - ptot
                    END IF
               END IF
          CASE KCPGUP                                       'PgUp
               IF top > min THEN
                    top = top - ptot
                    IF top < min THEN top = min
                    bot = top + ptot - 1
                    IF bot > max THEN bot = max
                    sel = top
                    refresh = TRUE
               END IF
          CASE KCPGDN                                       'PgDn
               IF bot < max THEN
                    top = top + ptot
                    bot = top + ptot - 1
                    IF bot > max THEN bot = max
                    sel = top
                    refresh = TRUE
               END IF
          CASE KCHOME                                       'Home
               sel = min
               IF top > min THEN
                    top = min
                    bot = top + ptot - 1
                    IF bot > max THEN bot = max
                    refresh = TRUE
               END IF
          CASE KCEND                                        'End
               sel = max
               IF bot < max THEN
                    bot = max
                    top = bot - ptot + 1
                    IF top < min THEN top = min
                    refresh = TRUE
               END IF
          CASE kTag                                         'Tag/Untag
               IF tag(sel) <> DISABLED THEN
                    tag(sel) = tag(sel) + 1
                    IF tag(sel) > TAGGED THEN tag(sel) = UNTAGGED
                    sel = sel + 1
               END IF
          CASE kAll                                         'Tag All
               FOR x = min TO max
                    IF tag(x) <> DISABLED THEN
                         tag(x) = TAGGED
                    END IF
               NEXT x
          CASE kNone                                        'Tag None
               FOR x = min TO max
                    IF tag(x) <> DISABLED THEN
                         tag(x) = UNTAGGED
                    END IF
               NEXT x
          CASE kToggle                                      'Toggle Tags
               FOR x = min TO max
                    IF tag(x) <> DISABLED THEN
                         tag(x) = tag(x) + 1
                         IF tag(x) > TAGGED THEN tag(x) = UNTAGGED
                    END IF
               NEXT x
          CASE ELSE                                         'Hotkey/Search
               FOR x = 13 TO (MINPARM - 1)
                    IF parm(x) = 0 THEN EXIT FOR
                    IF k = parm(x) THEN
                         EXIT DO
                    END IF
               NEXT x
               IF parm(8) THEN                              'Allow search?
                    l = LEN(search$)
                    IF k = KCBACKSPACE AND l THEN           'Backspace key
                         search$ = LEFT$(search$, l - 1)
                         refresh = TRUE
                    ELSEIF k >= 32 AND k <= 126 THEN        'Regular keys              
                         IF l <= wide - 2 AND l < 60 THEN
                              search$ = search$ + k$
                              l = l + 1
                              refresh = TRUE
                         END IF
                         found = min - 1
                         FOR x = sel TO max
                              IF LEFT$(UCASE$(choice$(x)), l) = search$ THEN
                                   found = x
                                   EXIT FOR
                              END IF
                         NEXT x
                         IF found < min THEN
                              FOR x = min TO sel - 1
                                   IF LEFT$(UCASE$(choice$(x)), l) = search$ THEN
                                        found = x
                                        EXIT FOR
                                   END IF
                              NEXT x
                         END IF
                         IF found >= min THEN
                              sel = found
                              IF sel < top OR sel > bot THEN
                                   top = min
                                   bot = top + ptot - 1
                              END IF
                              DO WHILE sel > bot
                                   top = top + ptot
                                   bot = top + ptot - 1
                              LOOP
                              IF bot > max THEN bot = max
                              refresh = TRUE
                         END IF
                    END IF
               END IF
     END SELECT

     IF sel < top THEN sel = bot
     IF sel > bot THEN sel = top

     IF k <> KCBACKSPACE AND (k < 32 OR k > 126) AND LEN(search$) THEN
          search$ = ""
          refresh = TRUE
     END IF

LOOP

x = SetCursor(oldcursor)                'Restore cursor to previous setting.
ERASE t$                                'Relinquish array memory.
ERASE temp

PickSome = sel                          'Return the current element number.

END FUNCTION

