DEFINT A-Z

' $INCLUDE: 'KEYCODE.INC'
' $INCLUDE: 'PARM.INC'
' $INCLUDE: 'TRUEFALS.INC'

DECLARE FUNCTION ViewFile (filename$, parm())
DECLARE FUNCTION VFilter$ (txt$)

'External functions:

DECLARE SUB BorderLines (parm())
DECLARE FUNCTION FileExist (f$)
DECLARE FUNCTION GetKey$ (parm())
DECLARE FUNCTION Imin (a, b)
DECLARE FUNCTION Imax (a, b)
DECLARE SUB InfoBox2 (msg$, parm())
DECLARE FUNCTION PadR$ (x$, n)
DECLARE SUB WipeArea (t, l, b, r)

FUNCTION VFilter$ (txt$)
'****************************************************************************
'This function is used only by ViewFile().  It replaces all non-text
' characters in a string with spaces.  This allows ViewFile() to work with
' files that have a few control codes in there with the regular text without
' crashing.
'****************************************************************************

x$ = txt$

FOR x = 1 TO LEN(x$)
     IF ASC(MID$(x$, x, 1)) < 32 THEN MID$(x$, x, 1) = " "
NEXT x

VFilter$ = x$

END FUNCTION

FUNCTION ViewFile (filename$, parm())
'****************************************************************************
'ViewFile() is a very simple text file viewer.  It works best on very small
' files.  It will return the KeyCode() of the key used to exit the function.
'
'    parm(1) = top row
'    parm(2) = bottom row
'    parm(3) = size of text buffer in screen pages
'    parm(4) = additional hotkeys...
'
'parm(3) represents the number of text pages in memory on either side of the
' one that is currently visible, i.e. you can hit PgUp or PgDn parm(3) times
' before the input file needs to be reread.  This value defaults to 1:  the
' page currently visible and one pagefull on either side of the current page
' are in memory at once.  There is no upper limit imposed by the function on
' this value so you need to be conservative when setting it.  A value of 1 to
' 4 should be sufficient for most small text files.  One page of text is the
' number of lines between parm(1) and parm(2), so be sure to allocate a
' bigger buffer if the space between these rows is small.
'
'parm(4 to MINPARM-1) can be specified as additional hotkeys that will exit
' the function.  See PickOne() or PickSome() for additional instructions on
' specifying hotkeys.
'
'You need to make sure that the filename passed to ViewFile() is actually a
' text file.  If this function tries to read a binary file, strange things
' happen.
'
'****************************************************************************

ViewFile = KCESC                        'Default return value.

tall = parm(2) - parm(1) - 1
buffer = parm(5) * tall
IF buffer < tall THEN buffer = tall

REDIM temp(1 TO MAXPARM)                'Make a temporary copy of parm()
FOR x = MINPARM TO MAXPARM
     temp(x) = parm(x)
NEXT x
temp(3) = 2

IF NOT FileExist(filename$) THEN
     InfoBox2 " " + filename$ + " not found. ", temp()
     ERASE temp
     EXIT FUNCTION
END IF

f = FREEFILE
OPEN filename$ FOR INPUT AS #f

IF LOF(f) = zero THEN
     CLOSE #f
     InfoBox2 " " + filename$ + " is empty. ", temp()
     ERASE temp
     EXIT FUNCTION
END IF

REDIM text$(1 TO 1)                     'Allow a buffer of +/- one screenfull

WipeArea parm(1), 1, parm(2), 80        'Find out how many lines in the file.
temp(1) = parm(1)
temp(2) = parm(2)
temp(3) = zero
BorderLines temp()
COLOR parm(FGH)
LOCATE parm(2), 2
PRINT "Reading...";
COLOR parm(FGN)
DO WHILE NOT EOF(f)                     'Find out how many lines in the file
     LINE INPUT #f, x$
     max = max + 1
LOOP
CLOSE #f

top = 1
bot = tall
tmin = 0
tmax = 0
temp(5) = 1
temp(6) = max

DO
     IF top < 1 THEN
          top = 1
          bot = tall
     END IF
     IF bot > max THEN
          bot = max
          top = bot - tall + 1
          IF top < 1 THEN top = 1
     END IF

     IF top < tmin OR bot > tmax THEN
          COLOR parm(FGH)
          LOCATE parm(2), 2
          PRINT "Reading...";
          COLOR parm(FGN)
          tmin = Imax(1, top - buffer)
          tmax = Imin(max, bot + buffer)
          REDIM text$(tmin TO tmax)
          f = FREEFILE
          OPEN filename$ FOR INPUT AS #f
          FOR x = 1 TO tmin - 1
               LINE INPUT #f, x$
          NEXT x
          FOR x = tmin TO tmax
               LINE INPUT #f, x$
               IF LEN(x$) > 80 THEN x$ = LEFT$(x$, 80)
               text$(x) = VFilter$(x$)
          NEXT x
          CLOSE #f
          WipeArea parm(1) + 1, 1, parm(2) - 1, 80
     END IF

     temp(3) = top
     temp(4) = bot
     BorderLines temp()

     row = parm(1) + 1
     FOR x = top TO bot
          LOCATE row, 1
          PRINT PadR$(text$(x), 80)
          row = row + 1
     NEXT x

     k = KeyCode(GetKey$(parm()))

     SELECT CASE k
          CASE KCESC
               EXIT DO
          CASE KCUP
               IF top > 1 THEN
                    top = top - 1
                    bot = bot - 1
               END IF
          CASE KCDOWN
               IF bot < max THEN
                    top = top + 1
                    bot = bot + 1
               END IF
          CASE KCPGUP
               IF top > 1 THEN
                    top = top - tall
                    bot = bot - tall
               END IF
          CASE KCPGDN
               IF bot < max THEN
                    top = top + tall
                    bot = bot + tall
               END IF
          CASE KCHOME
               IF top > 1 THEN
                    top = 1
                    bot = tall
               END IF
          CASE KCEND
               IF bot < max THEN
                    bot = max
                    top = bot - tall + 1
               END IF
          CASE ELSE
               FOR x = 4 TO MINPARM - 1
                    IF parm(x) = zero THEN
                         EXIT FOR
                    ELSEIF parm(x) = k THEN
                         EXIT DO
                    END IF
               NEXT x
     END SELECT
LOOP

ERASE text$, temp

ViewFile = k

END FUNCTION

